﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Models.Aqo
{
    public class VertexAccessor
    {
        public GetSingle                PX                      { get; private set; }
        public GetSingle                PY                      { get; private set; }
        public GetSingle                PZ                      { get; private set; }
        public GetSingle                NX                      { get; private set; }
        public GetSingle                NY                      { get; private set; }
        public GetSingle                NZ                      { get; private set; }
        public GetSingle                TX                      { get; private set; }
        public GetSingle                TY                      { get; private set; }
        public GetSingle                TZ                      { get; private set; }
        public GetSingle                BX                      { get; private set; }
        public GetSingle                BY                      { get; private set; }
        public GetSingle                BZ                      { get; private set; }
        public GetSingle                U0                      { get; private set; }
        public GetSingle                V0                      { get; private set; }
        public GetSingle                U1                      { get; private set; }
        public GetSingle                V1                      { get; private set; }
        public GetByte                  B0                      { get; private set; }
        public GetByte                  B1                      { get; private set; }
        public GetByte                  B2                      { get; private set; }
        public GetByte                  B3                      { get; private set; }
        public GetSingle                W0                      { get; private set; }
        public GetSingle                W1                      { get; private set; }
        public GetSingle                W2                      { get; private set; }
        public GetSingle                W3                      { get; private set; }

        public VertexAccessor(AFPFile aqo, VSET.VSETData vset, VTXL vtxl, VTXE vtxe)
        {
            var data    = vtxl._89BA;
            var stride  = (int)vset.VertSize;

            foreach(var i in vtxe.Data)
            {
                switch(i.Usage)
                {
                case 0:     // Position
                    AFPFile.Assert(i.Type == 3);
                    AFPFile.Assert(i.Index == 0);

                    PX  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 0 };
                    PY  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 4 };
                    PZ  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 8 };
                    break;

                case 1:     // BlendWeight
                    AFPFile.Assert(i.Type == 4);
                    AFPFile.Assert(i.Index == 0);

                    W0  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 0 };
                    W1  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 4 };
                    W2  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 8 };
                    W3  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+12 };
                    break;

                case 2:     // Normal
                    AFPFile.Assert(i.Type == 3);
                    AFPFile.Assert(i.Index == 0);

                    NX  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 0 };
                    NY  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 4 };
                    NZ  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 8 };
                    break;

                case 32:    // Tangent
                    AFPFile.Assert(i.Type == 3);
                    AFPFile.Assert(i.Index == 0);

                    TX  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 0 };
                    TY  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 4 };
                    TZ  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 8 };
                    break;

                case 33:    // Binormal
                    AFPFile.Assert(i.Type == 3);
                    AFPFile.Assert(i.Index == 0);

                    BX  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 0 };
                    BY  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 4 };
                    BZ  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 8 };
                    break;

                case 11:    // BlendIndices
                    AFPFile.Assert(i.Type == 7);
                    AFPFile.Assert(i.Index == 0);

                    B0  = new GetByte()   { Data= data, Stride= stride, Offset= (int)i.Offset+ 0 };
                    B1  = new GetByte()   { Data= data, Stride= stride, Offset= (int)i.Offset+ 1 };
                    B2  = new GetByte()   { Data= data, Stride= stride, Offset= (int)i.Offset+ 2 };
                    B3  = new GetByte()   { Data= data, Stride= stride, Offset= (int)i.Offset+ 3 };
                    break;

                case 16:
                    AFPFile.Assert(i.Type == 2);
                    AFPFile.Assert(i.Index == 0);

                    U0  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 0 };
                    V0  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 4 };
                    break;

                case 17:
                    AFPFile.Assert(i.Type == 2);
                    AFPFile.Assert(i.Index == 0);

                    U1  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 0 };
                    V1  = new GetSingle() { Data= data, Stride= stride, Offset= (int)i.Offset+ 4 };
                    break;

                default:
                    break;
                }
            }
        }
    }

    public abstract class Getter<T>
    {
        public byte[]                   Data                    { get; set; }
        public int                      Stride                  { get; set; }
        public int                      Offset                  { get; set; }
        public virtual T                this[int index]
        {
            get { return Get(index); }
            set { Set(index, value); }
        }

        public void Set(int index, byte[] value)
        {
            Array.Copy(value, 0, Data, Stride * index + Offset, value.Length);
        }

        public abstract T Get(int index);
        public abstract void Set(int index, T value);
    }

    public class GetSingle : Getter<float>
    {
        public override float Get(int index)
        {
            return BitConverter.ToSingle(Data, Stride * index + Offset);
        }

        public override void Set(int index, float value)
        {
            Set(index, BitConverter.GetBytes(value));
        }
    }

    public class GetByte : Getter<byte>
    {
        public override byte Get(int index)
        {
            return Data[Stride * index + Offset];
        }

        public override void Set(int index, byte value)
        {
            Data[Stride * index + Offset]   = value;
        }
    }
}
