﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml.Serialization;

namespace Models.Aqo
{
    using Root = XmlRootAttribute;
    using Elem = XmlElementAttribute;
    using Attr = XmlAttributeAttribute;
    using Ignore = XmlIgnoreAttribute;

    public class Config
    {
        public static string            FileName
        {
            get
            {
                var dir = Path.GetDirectoryName(typeof(Config).Assembly.Location);
                var file= Path.Combine(dir, "aqp2obj.xml");

                return file;
            }
        }

        [Elem("param")] public List<ConfigParam> Params         { get; set; }
        [Ignore] public Dictionary<string, string> ParamMap     { get; set; }

        public Config()
        {
            Params  = new List<ConfigParam>();
        }

        public void CreateDefault()
        {
            Params.Add(new ConfigParam() { Name= "X", Value= "+X" });
            Params.Add(new ConfigParam() { Name= "Y", Value= "-Z" });
            Params.Add(new ConfigParam() { Name= "Z", Value= "+Y" });
            Params.Add(new ConfigParam() { Name= "U",  Value= "U" });
            Params.Add(new ConfigParam() { Name= "V",  Value= "1-V" });
            Params.Add(new ConfigParam() { Name= "Magnify", Value= "100" });
            Params.Add(new ConfigParam() { Name= "Face", Value= "Front" });
            Params.Add(new ConfigParam() { Name= "Mode", Value= "0" });
        }

        public string Get(string name, string def)
        {
            string  value;

            return ParamMap.TryGetValue(name, out value) ? value : def;
        }

        public void Set(string name, string value)
        {
            ParamMap[name]  = value;
        }

        public static void ToFile(string file, Config data)
        {
            var xs  = new XmlSerializer(typeof(Config));

            using(var s= File.OpenWrite(file))
            {
                xs.Serialize(s, data);

                s.SetLength(s.Position);
            }
        }

        public static Config FromFile(string file)
        {
            if(!File.Exists(file))
            {
                var config  = new Config();

                config.CreateDefault();

                ToFile(file, config);
            }

            var xs  = new XmlSerializer(typeof(Config));

            using(var s= File.OpenRead(file))
            {
                var config      = (Config)xs.Deserialize(s);
                config.ParamMap = config.Params.ToDictionary(i => i.Name, i => i.Value);

                return config;
            }
        }
    }

    public class ConfigParam
    {
        [Attr("name")] public string    Name                    { get; set; }
        [Attr("value")] public string   Value                   { get; set; }
    }
}
